@@ -23,12 +23,12 @@ class Accountant:
         log.info(f"Bank Account Number:  {self.bank_account['account_id']}")
         log.info(f"Cash Book:  {self.DEFAULT_CASH_BOOK_ID}")
 
-    def get_book(self):
+    def get_book(self, timestamp: int = None):
         log.info("---------Get Book----------")
-        blocked_balance_book = self.ledger.get_book(book_id=self.bank_account['ledger_books']['blocked_balance']['id'])['balances']
-        main_balance_book = self.ledger.get_book(book_id=self.bank_account['ledger_books']['main_balance']['id'])['balances']
-        blocked_balance_deposit_book = self.ledger.get_book_balances(book_id=self.bank_account['ledger_books']['blocked_balance']['id'], metadata_filter={'operation': 'DEPOSIT'})
-        blocked_balance_withdraw_book = self.ledger.get_book_balances(book_id=self.bank_account['ledger_books']['blocked_balance']['id'], metadata_filter={'operation': 'WITHDRAW'})
+        blocked_balance_book = self.ledger.get_book(book_id=self.bank_account['ledger_books']['blocked_balance']['id'], timestamp=timestamp)['balances']
+        main_balance_book = self.ledger.get_book(book_id=self.bank_account['ledger_books']['main_balance']['id'], timestamp=timestamp)['balances']
+        blocked_balance_deposit_book = self.ledger.get_book_balances(book_id=self.bank_account['ledger_books']['blocked_balance']['id'], metadata_filter={'operation': 'DEPOSIT'}, timestamp=timestamp)
+        blocked_balance_withdraw_book = self.ledger.get_book_balances(book_id=self.bank_account['ledger_books']['blocked_balance']['id'], metadata_filter={'operation': 'WITHDRAW'}, timestamp=timestamp)
         book_balance = []
         for currency, amount in blocked_balance_book.items():
             if currency in main_balance_book:
@@ -52,16 +52,16 @@ class Accountant:
             book_balance.append(dict(balance))
         return book_balance
 
-    def get_book_balances(self, currency: str, metadata_filter: dict = {}):
+    def get_book_balances(self, currency: str, metadata_filter: dict = {}, timestamp: int = None):
         log.info("---------Balance----------")
         blocked_deposit_balance = self.ledger.get_book_balances(
             book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
-            asset_id=currency, metadata_filter={**{'operation': 'DEPOSIT'}, **metadata_filter})
+            asset_id=currency, metadata_filter={**{'operation': 'DEPOSIT'}, **metadata_filter}, timestamp=timestamp)
         blocked_withdraw_balance = self.ledger.get_book_balances(
             book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
-            asset_id=currency, metadata_filter={**{'operation': 'WITHDRAW'}, **metadata_filter})
+            asset_id=currency, metadata_filter={**{'operation': 'WITHDRAW'}, **metadata_filter}, timestamp=timestamp)
         main_balance = self.ledger.get_book_balances(book_id=self.bank_account['ledger_books']['main_balance']['id'],
-                                                     asset_id=currency, metadata_filter={**metadata_filter})
+                                                     asset_id=currency, metadata_filter={**metadata_filter}, timestamp=timestamp)
         return {'currency': currency,
                 'blocked_balance_deposit': blocked_deposit_balance.get(currency, "0"),
                 'blocked_balance_withdraw': blocked_withdraw_balance.get(currency, "0"),
@@ -69,65 +69,65 @@ class Accountant:
 
     def withdraw_block_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Block_Amount_Withdraw----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['main_balance']['id'],
             to_book_id=self.bank_account['ledger_books']['blocked_balance']['id'], asset_id=currency, value=value,
             memo=memo, metadata={**{"operation": "WITHDRAW"}, **metadata})
         log.info(operation)
         return operation
 
-    def deposit_block_amount(self, value: str, currency: str, memo: str):
+    def deposit_block_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Block_Amount_Deposit----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.DEFAULT_CASH_BOOK_ID['id'],
             to_book_id=self.bank_account['ledger_books']['blocked_balance']['id'], asset_id=currency, value=value,
             memo=memo,
-            metadata={'operation': 'DEPOSIT'})
+            metadata={**{'operation': 'DEPOSIT'}, **metadata})
         log.info(operation)
         return operation
 
-    def withdraw_unblock_amount(self, value: str, currency: str, memo: str):
+    def withdraw_unblock_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Unblock_Amount_Withdraw----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
             to_book_id=self.bank_account['ledger_books']['main_balance']['id'], asset_id=currency, value=value,
             memo=memo,
-            metadata={'operation': 'WITHDRAW'})
+            metadata={**{'operation': 'WITHDRAW'}, **metadata})
         log.info(operation)
         return operation
 
-    def deposit_unblock_amount(self, value: str, currency: str, memo: str):
+    def deposit_unblock_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Unblock_Amount_Deposit----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
             to_book_id=self.DEFAULT_CASH_BOOK_ID['id'], asset_id=currency, value=value, memo=memo,
-            metadata={'operation': 'DEPOSIT'})
+            metadata={**{'operation': 'DEPOSIT'}, **metadata})
         log.info(operation)
         return operation
 
-    def withdraw_amount(self, value: str, currency: str, memo: str):
+    def withdraw_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Withdraw_amount----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
             to_book_id=self.DEFAULT_CASH_BOOK_ID['id'],
-            asset_id=currency, value=value, memo=memo, metadata={'operation': 'WITHDRAW'})
+            asset_id=currency, value=value, memo=memo, metadata={**{'operation': 'WITHDRAW'}, **metadata})
         log.info(operation)
         return operation
 
-    def deposit_amount(self, value: str, currency: str, memo: str):
+    def deposit_amount(self, value: str, currency: str, memo: str, metadata: dict = {}):
         log.info("---------Deposit_amount----------")
-        operation = self.ledger.post_transfer_sync(
+        operation = self.ledger.post_transfer(
             from_book_id=self.bank_account['ledger_books']['blocked_balance']['id'],
             to_book_id=self.bank_account['ledger_books'][
                 'main_balance']['id'], asset_id=currency, value=value,
-            memo=memo, metadata={'operation': 'DEPOSIT'})
+            memo=memo, metadata={**{'operation': 'DEPOSIT'}, **metadata})
         log.info(operation)
         return operation
 
-    def block_trade_amount(self, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str):
+    def block_trade_amount(self, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str, metadata: dict = {}):
         log.info("---------Block_Trade_Amount----------")
-        withdraw_block_amount = self.withdraw_block_amount(value=from_amount, currency=from_currency, memo=memo)
-        deposit_block_amount = self.deposit_block_amount(value=to_amount, currency=to_currency, memo=memo)
+        withdraw_block_amount = self.withdraw_block_amount(value=from_amount, currency=from_currency, memo=memo, metadata=metadata)
+        deposit_block_amount = self.deposit_block_amount(value=to_amount, currency=to_currency, memo=memo, metadata=metadata)
         operation = {
             'withdraw_block_amount': withdraw_block_amount,
             'deposit_block_amount': deposit_block_amount
@@ -135,10 +135,10 @@ class Accountant:
         log.info(operation)
         return operation
 
-    def trade_amount(self, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str):
+    def trade_amount(self, from_currency: str, to_currency: str, from_amount: str, to_amount: str, memo: str, metadata: dict = {}):
         log.info("---------Trade_Amount----------")
-        withdraw_amount = self.withdraw_amount(value=from_amount, currency=from_currency, memo=memo)
-        deposit_amount = self.deposit_amount(value=to_amount, currency=to_currency, memo=memo)
+        withdraw_amount = self.withdraw_amount(value=from_amount, currency=from_currency, memo=memo, metadata=metadata)
+        deposit_amount = self.deposit_amount(value=to_amount, currency=to_currency, memo=memo, metadata=metadata)
         operation = {
             'withdraw_block_amount': withdraw_amount,
             'deposit_block_amount': deposit_amount
@@ -146,6 +146,20 @@ class Accountant:
         log.info(operation)
         return operation
 
+    def freeze_books(self):
+        log.info("---------Freezing Ledger Books----------")
+        blocked_book = self.ledger.freeze_book(self.bank_account['ledger_books']['blocked_balance']['id'])
+        main_book = self.ledger.freeze_book(self.bank_account['ledger_books']['main_balance']['id'])
+        log.info(blocked_book, main_book)
+        return main_book, blocked_book
+
+    def unfreeze_books(self):
+        log.info("---------Unfreezing Ledger Books----------")
+        blocked_book = self.ledger.unfreeze_book(self.bank_account['ledger_books']['blocked_balance']['id'])
+        main_book = self.ledger.unfreeze_book(self.bank_account['ledger_books']['main_balance']['id'])
+        log.info(blocked_book, main_book)
+        return main_book, blocked_book
+
     @staticmethod
     def create_book(account_id: str, ledger_host):
         log.info("---------New_ledger-------------")
